@@ -1,8 +1,9 @@
-import sys
 import io
 import re
 import struct
 import hashlib as md5
+import logging
+
 from .psparser import PSStackParser, PSSyntaxError, PSEOF, literal_name, LIT, KWD, STRICT
 from .pdftypes import (PDFException, PDFTypeError, PDFNotImplementedError, PDFStream, PDFObjRef,
     resolve1, decipher_all, int_value, str_value, list_value, dict_value, stream_value)
@@ -46,7 +47,7 @@ class PDFXRef(PDFBaseXRef):
         self.offsets = {}
         self.trailer = {}
 
-    def load(self, parser, debug=0):
+    def load(self, parser):
         while 1:
             try:
                 (pos, line) = parser.nextline()
@@ -56,7 +57,7 @@ class PDFXRef(PDFBaseXRef):
             if not line:
                 raise PDFNoValidXRef('Premature eof: %r' % parser)
             if line.startswith('trailer'):
-                parser.seek(pos)
+                parser.setpos(pos)
                 break
             f = line.strip().split(' ')
             if len(f) != 2:
@@ -76,8 +77,7 @@ class PDFXRef(PDFBaseXRef):
                 (pos, genno, use) = f
                 if use != 'n': continue
                 self.offsets[objid] = (int(genno), int(pos))
-        if 1 <= debug:
-            print('xref objects:', self.offsets, file=sys.stderr)
+        logging.debug('xref objects: %r', self.offsets)
         self.load_trailer(parser)
 
     KEYWORD_TRAILER = KWD('trailer')
@@ -94,18 +94,18 @@ class PDFXRef(PDFBaseXRef):
         self.trailer.update(dict_value(dic))
 
     PDFOBJ_CUE = re.compile(r'^(\d+)\s+(\d+)\s+obj\b')
+
     def load_fallback(self, parser, debug=0):
-        parser.seek(0)
+        parser.setpos(0)
         while 1:
             try:
                 (pos, line) = parser.nextline()
             except PSEOF:
                 break
             if line.startswith('trailer'):
-                parser.seek(pos)
+                parser.setpos(pos)
                 self.load_trailer(parser)
-                if 1 <= debug:
-                    print('trailer: %r' % self.get_trailer(), file=sys.stderr)
+                logging.debug('trailer: %r', self.get_trailer())
                 break
             m = self.PDFOBJ_CUE.match(line)
             if not m: continue
@@ -137,7 +137,7 @@ class PDFXRefStream(PDFBaseXRef):
     def __repr__(self):
         return '<PDFXRefStream: fields=%d,%d,%d>' % (self.fl1, self.fl2, self.fl3)
 
-    def load(self, parser, debug=0):
+    def load(self, parser):
         (_,objid) = parser.nexttoken() # ignored
         (_,genno) = parser.nexttoken() # ignored
         (_,kwd) = parser.nexttoken()
@@ -154,10 +154,9 @@ class PDFXRefStream(PDFBaseXRef):
         self.data = stream.get_data()
         self.entlen = self.fl1+self.fl2+self.fl3
         self.trailer = stream.attrs
-        if 1 <= debug:
-            print(('xref stream: objid=%s, fields=%d,%d,%d' %
-                             (', '.join(map(repr, self.objid_ranges)),
-                              self.fl1, self.fl2, self.fl3)), file=sys.stderr)
+        if logging.getLogger().getEffectiveLevel() <= logging.DEBUG:
+            logging.debug('xref stream: objid=%s, fields=%d,%d,%d',
+                ', '.join(map(repr, self.objid_ranges)), self.fl1, self.fl2, self.fl3)
 
     def get_trailer(self):
         return self.trailer
@@ -263,8 +262,6 @@ class PDFDocument:
     
     """
 
-    debug = 0
-
     def __init__(self, caching=True):
         self.caching = caching
         self.xrefs = []
@@ -376,8 +373,7 @@ class PDFDocument:
     def getobj(self, objid):
         if not self.xrefs:
             raise PDFException('PDFDocument is not initialized')
-        if 2 <= self.debug:
-            print('getobj: objid=%r' % (objid), file=sys.stderr)
+        # logging.debug('getobj: objid=%r', objid)
         if objid in self._cached_objs:
             genno = 0
             obj = self._cached_objs[objid]
@@ -427,7 +423,7 @@ class PDFDocument:
                 if isinstance(obj, PDFStream):
                     obj.set_objid(objid, 0)
             else:
-                self._parser.seek(index)
+                self._parser.setpos(index)
                 (_,objid1) = self._parser.nexttoken() # objid
                 (_,genno) = self._parser.nexttoken() # genno
                 (_,kwd) = self._parser.nexttoken()
@@ -450,8 +446,7 @@ class PDFDocument:
                         obj.set_objid(objid, genno)
                 except PSEOF:
                     return None
-            if 2 <= self.debug:
-                print('register: objid=%r: %r' % (objid, obj), file=sys.stderr)
+            # logging.debug('register: objid=%r: %r', objid, obj)
             if self.caching:
                 self._cached_objs[objid] = obj
         if self.decipher:
@@ -473,14 +468,12 @@ class PDFDocument:
                 if k in self.INHERITABLE_ATTRS and k not in tree:
                     tree[k] = v
             if tree.get('Type') is LITERAL_PAGES and 'Kids' in tree:
-                if 1 <= self.debug:
-                    print('Pages: Kids=%r' % tree['Kids'], file=sys.stderr)
+                logging.debug('Pages: Kids=%r', tree['Kids'])
                 for c in list_value(tree['Kids']):
                     for x in search(c, tree):
                         yield x
             elif tree.get('Type') is LITERAL_PAGE:
-                if 1 <= self.debug:
-                    print('Page: %r' % tree, file=sys.stderr)
+                logging.debug('Page: %r', tree)
                 yield (objid, tree)
         if 'Pages' not in self.catalog:
             return
@@ -603,14 +596,14 @@ class PDFParser(PSStackParser):
             # stream object
             ((_,dic),) = self.pop(1)
             dic = dict_value(dic)
-            objlen = 0
-            if not self.fallback:
-                try:
-                    objlen = int_value(dic['Length'])
-                except KeyError:
-                    if STRICT:
-                        raise PDFSyntaxError('/Length is undefined: %r' % dic)
-            self.seek(pos)
+            try:
+                objlen = int_value(dic['Length'])
+            except KeyError:
+                if STRICT:
+                    raise PDFSyntaxError('/Length is undefined: %r' % dic)
+                else:
+                    objlen = 0
+            self.setpos(pos)
             try:
                 (_, line) = self.nextline()  # 'stream'
             except PSEOF:
@@ -618,27 +611,18 @@ class PDFParser(PSStackParser):
                     raise PDFSyntaxError('Unexpected EOF')
                 return
             pos += len(line)
-            self.fp.seek(pos)
-            data = self.fp.read(objlen)
-            self.seek(pos+objlen)
-            while True:
-                next_chunk = self.fp.read(self.BUFSIZ)
-                if not next_chunk:
-                    if STRICT:
-                        raise PDFSyntaxError('Unexpected EOF')
-                    break
-                if b'endstream' in next_chunk:
-                    i = next_chunk.index(b'endstream')
-                    objlen += i
-                    data += next_chunk[:i]
-                    break
-                objlen += len(next_chunk)
-                data += next_chunk
-            self.seek(pos+objlen)
+            endpos = pos + objlen
+            if 'endstream' not in self.data[endpos:endpos+len('endstream')+2]:
+                r = re.compile(r'(\r\n|\r|\n)endstream')
+                m = r.search(self.data, pos)
+                if m is None:
+                    raise PDFSyntaxError("stream with no endstream")
+                endpos = m.start()
+            data = self.data[pos:endpos].encode('latin-1')
+            self.setpos(endpos)
+            self.nexttoken() # consume 'endstream'
             # XXX limit objlen not to exceed object boundary
-            if 2 <= self.debug:
-                print('Stream: pos=%d, objlen=%d, dic=%r, data=%r...' % \
-                      (pos, objlen, dic, data[:10]), file=sys.stderr)
+            # logging.debug('Stream: pos=%d, objlen=%d, dic=%r, data=%r...', pos, objlen, dic, data[:10])
             obj = PDFStream(dic, data, self.doc.decipher)
             self.push((pos, obj))
 
@@ -649,47 +633,41 @@ class PDFParser(PSStackParser):
 
     def find_xref(self):
         """Internal function used to locate the first XRef."""
-        # search the last xref table by scanning the file backwards.
-        prev = None
-        for line in self.revreadlines():
-            line = line.strip()
-            if 2 <= self.debug:
-                print('find_xref: %r' % line, file=sys.stderr)
-            if line == 'startxref': break
-            if line:
-                prev = line
-        else:
+        # the word 'startxref' followed by a newline followed by digits
+        re_startxref = re.compile(r'startxref\s*[\r\n]+\s*(\d+)', re.MULTILINE)
+        # try at the end, then try the whole file.
+        m = re_startxref.findall(self.data, len(self.data)-4096)
+        if not m:
+            m = re_startxref.findall(self.data)
+        if not m:
             raise PDFNoValidXRef('Unexpected EOF')
-        if 1 <= self.debug:
-            print('xref found: pos=%r' % prev, file=sys.stderr)
-        return int(prev)
-
+        logging.debug('xref found: pos=%r', m[-1])
+        return int(m[-1])
+    
     # read xref table
     def read_xref_from(self, start, xrefs):
         """Reads XRefs from the given location."""
-        self.seek(start)
+        self.setpos(start)
         self.reset()
         try:
             (pos, token) = self.nexttoken()
         except PSEOF:
             raise PDFNoValidXRef('Unexpected EOF')
-        if 2 <= self.debug:
-            print('read_xref_from: start=%d, token=%r' % (start, token), file=sys.stderr)
+        # logging.debug('read_xref_from: start=%d, token=%r', start, token)
         if isinstance(token, int):
             # XRefStream: PDF-1.5
-            self.seek(pos)
+            self.setpos(pos)
             self.reset()
             xref = PDFXRefStream()
-            xref.load(self, debug=self.debug)
+            xref.load(self)
         else:
             if token is self.KEYWORD_XREF:
                 self.nextline()
             xref = PDFXRef()
-            xref.load(self, debug=self.debug)
+            xref.load(self)
         xrefs.append(xref)
         trailer = xref.get_trailer()
-        if 1 <= self.debug:
-            print('trailer: %r' % trailer, file=sys.stderr)
+        logging.debug('trailer: %r', trailer)
         if 'XRefStm' in trailer:
             pos = int_value(trailer['XRefStm'])
             self.read_xref_from(pos, xrefs)
@@ -707,8 +685,7 @@ class PDFParser(PSStackParser):
             self.read_xref_from(pos, xrefs)
         except PDFNoValidXRef:
             # fallback
-            if 1 <= self.debug:
-                print('no xref, fallback', file=sys.stderr)
+            logging.debug('no xref, fallback')
             self.fallback = True
             xref = PDFXRef()
             xref.load_fallback(self)
